@@ -10,7 +10,7 @@ use File::Spec::Functions qw/catdir/;
 
 use vars qw{$VERSION @ISA};
 BEGIN {
-    $VERSION = '0.28';
+    $VERSION = '0.28_04';
     @ISA     = qw{Module::Install::Base};
 }
 
@@ -20,6 +20,7 @@ my $ccflags = "";
 if ($endianess == 4321) {
     $ccflags = " -DMONGO_BIG_ENDIAN=1 ";
 }
+$ccflags .= " -Wall ";
 
 sub mongo {
     my ($self, @mongo_vars) = @_;
